/********************************************************************

********************************************************************/

#include "predef.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <startnet.h>
#include <serial.h>
#include <iosys.h>
#include <utils.h>
#include <ip.h>
#include <tcp.h>
#include <autoupdate.h>
#include <string.h>
#include "clientweb.h"


#define APP_VERSION "Version 1.0 1/22/2003"
#define DEST_PORT (3334)			// default dest port
#define DEST_IPADDR "10.1.1.102"	// default dest ip addr
#define TX_BUFSIZE (4096)			// transmit buffer size
#define POST_BUFSIZE (4096)			// form post buffer size
#define VERIFY_KEY (0x18256050) 	// NV Settings key code

//----- function prototypes -----
extern "C"
{
	void WebDestPort(int sock, PCSTR url);
	void WebDestIp(int sock, PCSTR url);
}


//----- Global Vars -----
char TxBuffer[TX_BUFSIZE];
char gPostBuf[POST_BUFSIZE];

#define MAX_IPADDR_LEN (20)
struct NV_SettingsStruct		// non-volatile storage structure
{
	DWORD VerifyKey;
	WORD  nDestPort;
    char  szDestIpAddr[MAX_IPADDR_LEN];
};
NV_SettingsStruct NV_Settings;





/*-------------------------------------------------------------------
 Sends a message to the specified destination host.
 ------------------------------------------------------------------*/
void SendMsg(char *msg)
{
	char buf[80];

	int fd = connect(AsciiToIp(NV_Settings.szDestIpAddr), 0, 
   					  NV_Settings.nDestPort, TICKS_PER_SECOND * 5);
	if (fd < 0)
	{
		printf("Error connecting to: %s:%d\n", NV_Settings.szDestIpAddr, 
      			NV_Settings.nDestPort);
               
      // Add error HTML page 
	}
   else
   {
		int n = write(fd, msg, strlen(msg));
		printf("Wrote %d bytes: %s\n", n, TxBuffer);
	   close(fd);
	}
}


/*-------------------------------------------------------------------
 ------------------------------------------------------------------*/
void WebDestPort(int sock, PCSTR url)
{
	char buf[80];

	sprintf(buf, "VALUE=\"%d\" ", NV_Settings.nDestPort);
	writestring(sock, buf);   
}

/*-------------------------------------------------------------------
 ------------------------------------------------------------------*/
void WebDestIp(int sock, PCSTR url)
{
	char buf[80];

	sprintf(buf, "VALUE=\"%s\" ", NV_Settings.szDestIpAddr);
	writestring(sock, buf);   
}



/*-------------------------------------------------------------------
 Check NV Settings.  Assign default values if VerifyKey is not
 valid.
 ------------------------------------------------------------------*/
void CheckNVSettings()
{
	NV_SettingsStruct *pData = (NV_SettingsStruct *)GetUserParameters();
	NV_Settings.VerifyKey     = pData->VerifyKey;

	if (NV_Settings.VerifyKey != VERIFY_KEY)
	{
		NV_Settings.VerifyKey = VERIFY_KEY;
		NV_Settings.nDestPort = DEST_PORT;
		strncpy(NV_Settings.szDestIpAddr, DEST_IPADDR, MAX_IPADDR_LEN);

		SaveUserParameters(&NV_Settings, sizeof(NV_Settings));
	}
	else
	{
		NV_Settings.nDestPort = pData->nDestPort;
		strncpy(NV_Settings.szDestIpAddr, pData->szDestIpAddr, MAX_IPADDR_LEN);
	}
}


/*-------------------------------------------------------------------
 Handle HTTP Post

 WARNING WARNING WARNING     CAUTION CAUTION CAUTION
 The User data space is 8K in size.  The object we are storing is
 less than 200 bytes long, thus it is probably ok to make an automatic
 variable out of it.  It would NOT BE OK to make an 8K automatic
 variable.  The coices for doing this with an 8K object...

 First choice: make a global variable, this way the linker will 
 allocate space for it. All errors will be at link time not run time.

 Second choice: Increase the HTTP stack size in constants.h and
 recompile the whole system directory.

 Third choice: use malloc and free.  The only problem is what do 
 you do if malloc fails?
 ------------------------------------------------------------------*/
int MyDoPost(int sock, char * url, char * pData, char * rxBuffer)
{
   int max_chars = 40;
   char *buf = gPostBuf;	// post buffer is global

	iprintf(pData);	// print all data sent from app

	if (ExtractPostData("SendMessage", pData, buf, max_chars) > 0) 
   {
   	iprintf("Parsed Send Message Submit\r\n");
		if (ExtractPostData("tfMessage", pData, buf, max_chars) > 0) 
   	{
			iprintf("Sending Message: %s\r\n", buf);
         SendMsg(buf);
      }
   }

	if (ExtractPostData("SubmitNewSettings", pData, buf, max_chars) > 0) 
   {
		// Handle Destination Port Number
		if (ExtractPostData("tfDestPort", pData, buf, max_chars) == -1)
			iprintf("Error reading post data: tfDestPort\r\n");
	   else
			NV_Settings.nDestPort = (WORD)atoi(buf);


		// Handle Destination IP Address
		if (ExtractPostData("tfDestIpAddr", pData, buf, max_chars) == -1)
			iprintf( "Error reading post data: tfDestIpAddr\r\n");
	   else
	   {
	   	iprintf("Changing DestIpAddr to: %s\r\n", buf);
	   	strncpy(NV_Settings.szDestIpAddr, buf, MAX_IPADDR_LEN);
	   }


		// Now to store it in flash.  
	   // WARNING: If new settings are added, remember to add them to
	   //          NV Settings default initalization!
		if (ExtractPostData("SubmitNewSettings", pData, buf, max_chars) > 0)
	   {
		   NV_Settings.VerifyKey = VERIFY_KEY;
			if ( SaveUserParameters(&NV_Settings, sizeof(NV_Settings)) != 0 )
			{
		  		iprintf( "New Settings Saved\r\n");
			}
		   else
		      iprintf( "ERROR: Could not save new settings\r\n");
	   }
   }
 
	// We have to respond to the post with a new HTML page. In
   // this case we will redirect so the browser will go to 
   // that URL for the response.
	RedirectResponse(sock, "index.htm");

	return 0;
}


/*-------------------------------------------------------------------

 ------------------------------------------------------------------*/
void RegisterPost()
{
	SetNewPostHandler(MyDoPost);
}
